package MicroLang;

import java.io.LineNumberReader;
import java.io.IOException;

public class MicroScanner {
  private final LineNumberReader inFile;
  private StringBuffer restOfCurrentLine;

  public MicroScanner(LineNumberReader input) throws IOException {
    System.out.println("Starting lexical analysis ... ");
    inFile = input;
    restOfCurrentLine = new StringBuffer(inFile.readLine());
  }

    private static boolean IsMicLetter(char CharacterToTest) {
    if (Character.isLetter(CharacterToTest) &&
       ('\u0000' <= CharacterToTest && CharacterToTest <= '\u00FF'))
      return true;
    else
      return false;
  }

  private static boolean IsMicDigit(char CharacterToTest) {
    if (Character.isDigit(CharacterToTest) &&
       ('\u0000' <= CharacterToTest && CharacterToTest <= '\u00FF'))
      return true;
    else
      return false;
  }

  private static boolean IsMicLetterOrDigit(char CharacterToTest) {
    return (IsMicLetter(CharacterToTest)||IsMicDigit(CharacterToTest));
  }


  public MicroToken getNextToken() throws IOException, MicroLexicalError {
    int ident_or_lit_len=0;
    StringBuffer ident_or_lit_name = new StringBuffer("");
    char currentChar;
    if(restOfCurrentLine.length()==0) {
      try {
        while (restOfCurrentLine.length()==0)
          restOfCurrentLine=new StringBuffer(inFile.readLine());
      }
      catch(NullPointerException npe) {
        System.out.println("EOF");
        return new MicroToken(MicroToken.EOTtoken, null);
      }
    }
    if(restOfCurrentLine.length()!=0) {
      currentChar=restOfCurrentLine.charAt(0);

      //identifier
      if (IsMicLetter(currentChar)) {
        ident_or_lit_name.append(currentChar);
        restOfCurrentLine.deleteCharAt(0);
        while (restOfCurrentLine.length()!=0) {
          currentChar=restOfCurrentLine.charAt(0);
          if(currentChar=='_'){
            if (restOfCurrentLine.length()<2)
              throw new MicroLexicalError("lexical error in line " +
                                          inFile.getLineNumber() +
                                          ": '_' at the end of Identifier");
            else {
              ident_or_lit_name.append(currentChar);
              restOfCurrentLine.deleteCharAt(0);
              currentChar=restOfCurrentLine.charAt(0);
              if(!(IsMicLetterOrDigit(currentChar)))
                throw new MicroLexicalError("lexical error in line " +
                                            inFile.getLineNumber() +
                                            ": '_' at the end of Identifier");
            }
          }
          if(!IsMicLetterOrDigit(currentChar))
            return new MicroToken(MicroToken.Ident,
                                  ident_or_lit_name.toString());

          ident_or_lit_name.append(currentChar);
          restOfCurrentLine.deleteCharAt(0);
        }
        return new MicroToken(MicroToken.Ident, ident_or_lit_name.toString());
      }

      //INT_LIT
      if (IsMicDigit(currentChar)) {
        ident_or_lit_name.append(currentChar);
        while (restOfCurrentLine.length()!=0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='_') {
            if (restOfCurrentLine.length()<2)
              throw new MicroLexicalError("lexical error in line " +
                                          inFile.getLineNumber() +
                                          ": '_' at the end of Int Literal");
            else {
              restOfCurrentLine.deleteCharAt(0);
              currentChar=restOfCurrentLine.charAt(0);
              if (!IsMicDigit(currentChar))
                throw new MicroLexicalError("lexical error in line " +
                                            inFile.getLineNumber() +
                                            ": '_' at the end of Int Literal");
            }
          }

          if (!IsMicDigit(currentChar))
            return new MicroToken(MicroToken.IntLit,
                                  ident_or_lit_name.toString());
          ident_or_lit_name.append(currentChar);
          restOfCurrentLine.deleteCharAt(0);
        }
        return new MicroToken(MicroToken.IntLit,
                              ident_or_lit_name.toString());
      }

      switch (currentChar) {

      //COLON or ASSIGN
      case ':':
        restOfCurrentLine.deleteCharAt(0);
        if (restOfCurrentLine.length()!= 0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='=') {
            restOfCurrentLine.deleteCharAt(0);
            return new MicroToken(MicroToken.Assign, null);
          }
        }
        return new MicroToken(MicroToken.Colon, null);

      //SEMICOLON
      case ';':
        restOfCurrentLine.deleteCharAt(0);
        return new MicroToken(MicroToken.SemiColon, null);

      //LEFT BRACKET
      case '(':
        restOfCurrentLine.deleteCharAt(0);
        return new MicroToken(MicroToken.LeftParenthesis, null);

      //RIGHT BRACKET
      case ')':
        restOfCurrentLine.deleteCharAt(0);
        return new MicroToken(MicroToken.RightParenthesis, null);

      //PLUS
      case '+':
        restOfCurrentLine.deleteCharAt(0);
        return new MicroToken(MicroToken.Plus, null);

      //EQUALS or IMPL
      case '=':
        restOfCurrentLine.deleteCharAt(0);
        if (restOfCurrentLine.length()!= 0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='>') {
            restOfCurrentLine.deleteCharAt(0);
            return new MicroToken(MicroToken.Impl, null);
          }
        }
        return new MicroToken(MicroToken.Equal, null);

      //LESS or LESS EQUAL or EQUIV
      case '<':
        restOfCurrentLine.deleteCharAt(0);
        if (restOfCurrentLine.length()!= 0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='=') {
            restOfCurrentLine.deleteCharAt(0);
            if (restOfCurrentLine.length()!= 0) {
              currentChar=restOfCurrentLine.charAt(0);
              if (currentChar=='>') {
                restOfCurrentLine.deleteCharAt(0);
                return new MicroToken(MicroToken.Equiv, null);
              }
            }
            return new MicroToken(MicroToken.LessEqual, null);
          }
        }
        return new MicroToken(MicroToken.Less, null);

      //GREATER or GREATER EQUAL
      case '>':
        restOfCurrentLine.deleteCharAt(0);
        if (restOfCurrentLine.length()!= 0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='=') {
            restOfCurrentLine.deleteCharAt(0);
            return new MicroToken(MicroToken.GreaterEqual, null);
          }
        }
      return new MicroToken(MicroToken.Greater, null);

      //MINUS or COMMENT
      case '-':
        restOfCurrentLine.deleteCharAt(0);
        if (restOfCurrentLine.length()!= 0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='-') {
            restOfCurrentLine.delete(0, restOfCurrentLine.length());
            return new MicroToken(MicroToken.WhiteSpace, null);
          }
        }
        return new MicroToken(MicroToken.Minus, null);

      //TIMES or POWER
      case '*':
        restOfCurrentLine.deleteCharAt(0);
        if (restOfCurrentLine.length()!= 0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='*') {
            restOfCurrentLine.deleteCharAt(0);
            return new MicroToken(MicroToken.Power, null);
          }
        }
        return new MicroToken(MicroToken.Times, null);

      //DIV or NOT EQUALS
      case '/':
      restOfCurrentLine.deleteCharAt(0);
      if (restOfCurrentLine.length()!= 0) {
        currentChar=restOfCurrentLine.charAt(0);
        if (currentChar=='=') {
          restOfCurrentLine.deleteCharAt(0);
          return new MicroToken(MicroToken.NotEqual, null);
        }
      }
      return new MicroToken(MicroToken.Div, null);

      //DOTS
      case '.':
        restOfCurrentLine.deleteCharAt(0);
        if (restOfCurrentLine.length()!= 0) {
          currentChar=restOfCurrentLine.charAt(0);
          if (currentChar=='.') {
            restOfCurrentLine.deleteCharAt(0);
            return new MicroToken(MicroToken.Dots, null);
          }
        }
        throw new MicroLexicalError("lexical Error in line" +
                                    inFile.getLineNumber() +
                                    ": Two '.' expected");

      //WHITESPACE
      case ' ':
      case '\t':
        restOfCurrentLine.deleteCharAt(0);
        return new MicroToken(MicroToken.WhiteSpace, null);
      case '\u0003':
        return new MicroToken(MicroToken.EOTtoken, null);

      // rest is Lexical Error.
      default:
        throw new MicroLexicalError("lexical error in line " +
                                    inFile.getLineNumber()+
                                    ": wrong character");
      }
    }
    return new MicroToken(MicroToken.EOTtoken, null);

  }

}
