//----------------------------------------------------------------------------
// Dateiname     : MicroToken.java
// Erstellt am   : 20.04.2002
// Beschreibung  : Objekttyp fr die einzelnen lexikalischen Einheiten.
//                   - Teil des Package MicroLang
//                   - Unterscheidung von Identifiern und Reservierten 
//                     Bezeichnern
//                   - Jedes Objekt enthlt den Tokentyp und den zugehrigen
//                     Stringwert
//                   - Die Methode tokenString gibt zu einer bekannten TokenID
//                     vom Typ integer den zugehrigen Stringwert zurck.
//                   - Alle berprfungen wurden mit hilfe von Stringarrays 
//                     durchgefhrt
//----------------------------------------------------------------------------
// Autor         : carsten Freining
// Matrikel#     : xxxxx
// Aufgabe       : Compilerbau SS2002 3.1
//----------------------------------------------------------------------------

package MicroLang;

public class MicroToken {
  private int tokenKind;
  private String tokenValue;

  public MicroToken(int kind, String value) {
    tokenKind = kind;
    tokenValue = value;
    if (tokenKind == Ident) {
      tokenKind = lookupKind(tokenValue);
    }
  }

  public MicroToken() {}

  private int lookupKind(String IdString) {
    for(int i=0;i<ResIdents.length;i++) {
      if (IdString.equalsIgnoreCase(ResIdents[i]))
        return i;
    }
    return Ident;
  }

  public int getTokenKind() {
    return tokenKind;
  }

  public String getTokenValue() {
    return tokenValue;
  }

  // reserved identifiers
  public final static int AndSym            =  0;
  public final static int AsgSym            =  1;
  public final static int BlockSym          =  2;
  public final static int VarSym            =  3;
  public final static int ElseSym           =  4;
  public final static int EndSym            =  5;
  public final static int ForSym            =  6;
  public final static int IfSym             =  7;
  public final static int InSym             =  8;
  public final static int LoopSym           =  9;
  public final static int NotSym            = 10;
  public final static int OrSym             = 11;
  public final static int ReadSym           = 12;
  public final static int ThenSym           = 13;
  public final static int WhileSym          = 14;
  public final static int WriteSym          = 15;

  // special
  public final static int Colon             = 16;
  public final static int SemiColon         = 17;
  public final static int LeftParenthesis   = 18;
  public final static int RightParenthesis  = 19;

  // operators
  public final static int Assign            = 20;
  public final static int Equiv             = 21;
  public final static int Impl              = 22;
  public final static int Equal             = 23;
  public final static int NotEqual          = 24;
  public final static int Less              = 25;
  public final static int LessEqual         = 26;
  public final static int Greater           = 27;
  public final static int GreaterEqual      = 28;
  public final static int Plus              = 29;
  public final static int Minus             = 30;
  public final static int Times             = 31;
  public final static int Div               = 32;
  public final static int Power             = 33;
  public final static int Dots              = 34;

  public final static int Ident             = 35;
  public final static int IntLit            = 36;

  public final static int EOTtoken          = 100;
  public final static int WhiteSpace        = 200;

  // string respresentation of the reserved identifiers
  private final static String[] ResIdents =
    { "AND", "ASG", "BLOCK", "VAR", "ELSE", "END", "FOR", "IF",
      "IN", "LOOP", "NOT", "OR", "READ", "THEN", "WHILE", "WRITE" };

  private final static String[] TokenName =
    { "AndSym", "AsgSym", "BlockSym", "VarSym", "ElseSym", "EndSym", "ForSym",
      "IfSym", "InSym", "LoopSym", "NotSym", "OrSym", "ReadSym", "ThenSym",
      "WhileSym", "WriteSym", "Colon", "SemiColon", "LeftParenthesis",
      "RightParenthesis", "Assign", "Equiv", "impl", "Equal", "NotEqual",
      "Less", "LessEqual", "Greater", "GreaterEqual", "Plus", "Minus",
      "Times", "Div", "Power", "Dots", "Ident", "IntLit" };

  public static String tokenString(int kind) {
    if (kind==100) return "EOTtoken";
    if (kind==200) return "WhiteSpace";
    if ((kind>=0) && (kind <=36))
      return TokenName[kind];
    else
      return ("No Valid Token");

  }

  public void print() {
    System.out.println(tokenString(tokenKind) + " " + tokenValue);
  }
}
